@@ -8,7 +8,7 @@
  *
  *    In some countries OX, OX Open-Xchange, open xchange and OXtender
  *    as well as the corresponding Logos OX Open-Xchange and OX are registered
- *    trademarks of the Open-Xchange, Inc. group of companies.
+ *    trademarks of the OX Software GmbH group of companies.
  *    The use of the Logos is not covered by the GNU General Public License.
  *    Instead, you are allowed to use these Logos according to the terms and
  *    conditions of the Creative Commons License, Version 2.5, Attribution,
@@ -28,7 +28,7 @@
  *    http://www.open-xchange.com/EN/developer/. The contributing author shall be
  *    given Attribution for the derivative code and a license granting use.
  *
- *     Copyright (C) 2004-2015 Open-Xchange, Inc.
+ *     Copyright (C) 2016-2020 OX Software GmbH
  *     Mail: info@open-xchange.com
  *
  *
@@ -51,17 +51,32 @@ package com.openexchange.saml.http;
 
 import java.io.IOException;
 import java.io.PrintWriter;
+import java.util.Collection;
+import java.util.Map;
 import javax.servlet.ServletException;
+import javax.servlet.http.Cookie;
 import javax.servlet.http.HttpServletRequest;
 import javax.servlet.http.HttpServletResponse;
 import org.slf4j.Logger;
 import org.slf4j.LoggerFactory;
+import com.openexchange.ajax.login.HashCalculator;
+import com.openexchange.ajax.login.LoginConfiguration;
+import com.openexchange.ajax.login.LoginTools;
 import com.openexchange.exception.OXException;
+import com.openexchange.groupware.notify.hostname.HostnameService;
 import com.openexchange.java.Charsets;
+import com.openexchange.java.Strings;
+import com.openexchange.saml.SAMLConfig;
+import com.openexchange.saml.SAMLSessionParameters;
 import com.openexchange.saml.SAMLWebSSOProvider;
+import com.openexchange.saml.impl.LoginConfigurationLookup;
 import com.openexchange.saml.spi.ExceptionHandler;
+import com.openexchange.saml.tools.SAMLLoginTools;
+import com.openexchange.server.ServiceLookup;
 import com.openexchange.session.Session;
+import com.openexchange.sessiond.SessionFilter;
 import com.openexchange.sessiond.SessiondService;
+import com.openexchange.tools.servlet.http.Cookies;
 import com.openexchange.tools.servlet.http.Tools;
 
 
@@ -77,23 +92,32 @@ public class InitService extends SAMLServlet {
 
     private static final long serialVersionUID = -4022982444417155759L;
 
-    private final SessiondService sessiondService;
+    private final SAMLConfig config;
+
+    private final LoginConfigurationLookup loginConfigurationLookup;
+
+    private final ServiceLookup services;
 
 
     /**
      * Initializes a new {@link InitService}.
+     * @param config
      * @param provider
      * @param exceptionHandler
-     * @param sessiondService
+     * @param loginConfigurationLookup
+     * @param services
      */
-    public InitService(SAMLWebSSOProvider provider, ExceptionHandler exceptionHandler, SessiondService sessiondService) {
+    public InitService(SAMLConfig config, SAMLWebSSOProvider provider, ExceptionHandler exceptionHandler, LoginConfigurationLookup loginConfigurationLookup,
+        ServiceLookup services) {
         super(provider, exceptionHandler);
-        this.sessiondService = sessiondService;
+        this.config = config;
+        this.loginConfigurationLookup = loginConfigurationLookup;
+        this.services = services;
     }
 
     @Override
     protected void doGet(HttpServletRequest httpRequest, HttpServletResponse httpResponse) throws ServletException, IOException {
-        Tools.removeCachingHeader(httpResponse);
+        Tools.disableCaching(httpResponse);
 
         String flow = httpRequest.getParameter("flow");
         if (flow == null) {
@@ -105,7 +129,10 @@ public class InitService extends SAMLServlet {
         try {
             String redirectURI;
             if (flow.equals("login") || flow.equals("relogin")) {
-                redirectURI = provider.buildAuthnRequest(httpRequest, httpResponse);
+                redirectURI = tryAutoLogin(httpRequest, httpResponse);
+                if (redirectURI == null) {
+                    redirectURI = provider.buildAuthnRequest(httpRequest, httpResponse);
+                }
             } else if (flow.equals("logout")) {
                 String sessionId = httpRequest.getParameter("session");
                 if (sessionId == null) {
@@ -114,6 +141,7 @@ public class InitService extends SAMLServlet {
                     return;
                 }
 
+                SessiondService sessiondService = services.getService(SessiondService.class);
                 Session session = sessiondService.getSession(sessionId);
                 if (session == null) {
                     LOG.debug("Received SAML init request with invalid session parameter '{}'", sessionId);
@@ -146,4 +174,47 @@ public class InitService extends SAMLServlet {
         }
     }
 
+    private String tryAutoLogin(HttpServletRequest httpRequest, HttpServletResponse httpResponse) throws OXException {
+        Cookie samlCookie = null;
+        if (config.isAutoLoginEnabled()) {
+            LoginConfiguration loginConfiguration = loginConfigurationLookup.getLoginConfiguration();
+            String hash = HashCalculator.getInstance().getHash(httpRequest, LoginTools.parseUserAgent(httpRequest), LoginTools.parseClient(httpRequest, false, loginConfiguration.getDefaultClient()));
+            Map<String, Cookie> cookies = Cookies.cookieMapFor(httpRequest);
+            samlCookie = cookies.get(SAMLLoginTools.AUTO_LOGIN_COOKIE_PREFIX + hash);
+            if (samlCookie != null) {
+                SessiondService sessiondService = services.getService(SessiondService.class);
+                Collection<String> sessions = sessiondService.findSessions(SessionFilter.create("(" + SAMLSessionParameters.SESSION_COOKIE + "=" + samlCookie.getValue() + ")"));
+                if (sessions.size() > 0) {
+                    Session session = sessiondService.getSession(sessions.iterator().next());
+                    if (session == null) {
+                        LOG.debug("Found no session for SAML auto-login cookie '{}' with value '{}'", samlCookie.getName(), samlCookie.getValue());
+                    } else {
+                        try {
+                            LOG.debug("Found session '{}' for SAML auto-login cookie '{}' with value '{}'", session.getSessionID(), samlCookie.getName(), samlCookie.getValue());
+                            SAMLLoginTools.validateSession(httpRequest, session, hash, loginConfiguration);
+                            String uiWebPath = httpRequest.getParameter(SAMLLoginTools.PARAM_LOGIN_PATH);
+                            if (Strings.isEmpty(uiWebPath)) {
+                                uiWebPath = loginConfiguration.getUiWebPath();
+                            }
+                            return SAMLLoginTools.buildAbsoluteFrontendRedirectLocation(httpRequest, session, uiWebPath, services.getOptionalService(HostnameService.class));
+                        } catch (OXException e) {
+                            LOG.debug("Ignoring SAML auto-login attempt due to failed IP or secret check", e);
+                        }
+                    }
+                } else {
+                    LOG.debug("Found no session for SAML auto-login cookie '{}' with value '{}'", samlCookie.getName(), samlCookie.getValue());
+                }
+            }
+        }
+
+        if (samlCookie != null) {
+            // cookie exists but no according session was found => remove it
+            Cookie toRemove = (Cookie) samlCookie.clone();
+            toRemove.setMaxAge(0);
+            httpResponse.addCookie(toRemove);
+        }
+
+        return null;
+    }
+
 }
