@@ -8,7 +8,7 @@
  *
  *    In some countries OX, OX Open-Xchange, open xchange and OXtender
  *    as well as the corresponding Logos OX Open-Xchange and OX are registered
- *    trademarks of the Open-Xchange, Inc. group of companies.
+ *    trademarks of the OX Software GmbH group of companies.
  *    The use of the Logos is not covered by the GNU General Public License.
  *    Instead, you are allowed to use these Logos according to the terms and
  *    conditions of the Creative Commons License, Version 2.5, Attribution,
@@ -28,7 +28,7 @@
  *    http://www.open-xchange.com/EN/developer/. The contributing author shall be
  *    given Attribution for the derivative code and a license granting use.
  *
- *     Copyright (C) 2004-2015 Open-Xchange, Inc.
+ *     Copyright (C) 2016-2020 OX Software GmbH
  *     Mail: info@open-xchange.com
  *
  *
@@ -60,13 +60,13 @@ import com.openexchange.authentication.Authenticated;
 import com.openexchange.exception.OXException;
 import com.openexchange.saml.SAMLConfig;
 import com.openexchange.saml.SAMLExceptionCode;
+import com.openexchange.saml.state.AuthnRequestInfo;
 import com.openexchange.saml.state.StateManagement;
 import com.openexchange.saml.validation.AssertionValidator;
 import com.openexchange.saml.validation.ResponseValidator;
 import com.openexchange.saml.validation.StrictValidationStrategy;
 import com.openexchange.saml.validation.ValidationStrategy;
 import com.openexchange.session.reservation.EnhancedAuthenticated;
-import com.openexchange.session.reservation.Enhancer;
 
 /**
  * A {@link SAMLBackend} must be implemented and registered as OSGi service to enable
@@ -139,6 +139,19 @@ public interface SAMLBackend {
      */
     AuthenticationInfo resolveAuthnResponse(Response response, Assertion assertion) throws OXException;
 
+    /**
+     * Resolves an authentication response based on the bearer assertion that was determined and validated
+     * by the validation strategy.
+     *
+     * @param response The SAML response
+     * @param bearerAssertion The bearer assertion whose subject shall be mapped to a principal
+     * @param requestInfo The AuthnRequestInfo
+     * @return The authentication information
+     * @throws OXException If the principal cannot be resolved
+     * @see SAMLExceptionCode#UNKNOWN_USER
+     */
+    AuthenticationInfo resolveAuthnResponse(Response response, Assertion bearerAssertion, AuthnRequestInfo requestInfo) throws OXException;
+
     /**
      * Resolves a logout request and determines which sessions are to be terminated. This method is only
      * called if single logout is activated. Otherwise you can simply return <code>null</code>.
@@ -166,11 +179,10 @@ public interface SAMLBackend {
      * response. Use {@link EnhancedAuthenticated} to wrap the given {@link Authenticated} instance and add your
      * customizations.
      *
-     * @param authenticated The authenticated prepared by the 'redeemReservation' login action
+     * @param authenticated The authenticated prepared by the 'samlLogin' login action
      * @param properties The properties that were returned as part of {@link AuthenticationInfo} from {@link #resolveAuthnResponse(Response, Assertion)}
      * @return The enhanced {@link Authenticated}. If you don't need to adjust anything, simply return <code>null</code> here
      * @see EnhancedAuthenticated
-     * @see Enhancer
      */
     Authenticated enhanceAuthenticated(Authenticated authenticated, Map<String, String> properties);
 
